package jpa.test;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import jpa.test.entities.Document;
import jpa.test.entities.DocumentVersion;
import jpa.test.entities.Person;
import jpa.test.entities.Template;

import org.hibernate.Query;
import org.hibernate.Session;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class BugTest {
    
    private EntityManagerFactory emf;
    
    private Person p;
    
    @Before
    public void setup() {
        emf = Persistence.createEntityManagerFactory("TestPU");
        EntityManager em = emf.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        tx.begin();
        
        Document a = new Document("Aoc");
        em.persist(a);

        p = new Person();
        p.setName("Joe");
        em.persist(p);
        
        Template t = new Template();
        em.persist(t);
        
        DocumentVersion a1 = new DocumentVersion();
        a1.setDocument(a);
        a1.getTemplatesByOwner().put(p, t);
        em.persist(a1);
        
        tx.commit();
        em.close();
    }
    
    @After
    public void tearDown() {
        if (emf.isOpen()) {
            emf.close();
        }
    }
    
    @Test
    public void testKeyName() {
    	TestStatementInspector statementInspector = new TestStatementInspector();
        EntityManager em = emf.createEntityManager();
        Session session = em.unwrap(Session.class).getSessionFactory().withOptions().statementInspector(statementInspector).openSession();
        
        Query query = session.createQuery(
            "SELECT KEY(templatesByOwner).name FROM Document d "
            + "LEFT JOIN d.documentVersions versions "
            + "LEFT JOIN versions.templatesByOwner templatesByOwner");
        
        /**
         * Note: The generated SQL query needs to perform a join on Person in order to retrieve the name.
         */
        List<String> results = (List<String>) query.list();
        
        Assert.assertEquals(1, results.size());
        Assert.assertEquals(p.getName(), results.get(0));
        Assert.assertTrue(statementInspector.getSql().contains(" Person "));
    }
    
    @Test
    public void testKeyId() {
    	TestStatementInspector statementInspector = new TestStatementInspector();
        EntityManager em = emf.createEntityManager();
        Session session = em.unwrap(Session.class).getSessionFactory().withOptions().statementInspector(statementInspector).openSession();
        
        Query query = session.createQuery(
            "SELECT KEY(templatesByOwner).id FROM Document d "
            + "LEFT JOIN d.documentVersions versions "
            + "LEFT JOIN versions.templatesByOwner templatesByOwner");
        
        /**
         * Note: The generated SQL query should NOT perform a join on Person but should take the ID from
         * the KEY column in the join table instead.
         */
        List<Long> results = (List<Long>) query.list();
        
        Assert.assertEquals(1, results.size());
        Assert.assertEquals(p.getId(), results.get(0));
        Assert.assertFalse(statementInspector.getSql().contains(" Person "));
    }
    
}
